@@ -28,19 +28,41 @@
 SG_USING_STD(vector);
 
 
+
+////////////////////////////////////////////////////////////////////////
+// Global state
+////////////////////////////////////////////////////////////////////////
+static bool
+model_filter = true;
+
 
 ////////////////////////////////////////////////////////////////////////
 // Static utility functions.
 ////////////////////////////////////////////////////////////////////////
 
+static int
+model_filter_callback (ssgEntity * entity, int mask)
+{
+  return model_filter ? 1 : 0;
+}
+
 /**
  * Callback to update an animation.
  */
 static int
 animation_callback (ssgEntity * entity, int mask)
 {
-    ((SGAnimation *)entity->getUserData())->update();
-    return true;
+    return ((SGAnimation *)entity->getUserData())->update();
+}
+
+/**
+ * Callback to restore the state after an animation.
+ */
+static int
+restore_callback (ssgEntity * entity, int mask)
+{
+    ((SGAnimation *)entity->getUserData())->restore();
+    return 1;
 }
 
 
@@ -132,6 +154,10 @@ sgMakeAnimation( ssgBranch * model,
     animation = new SGBlendAnimation(prop_root, node);
   } else if (!strcmp("alpha-test", type)) {
     animation = new SGAlphaTestAnimation(node);
+  } else if (!strcmp("flash", type)) {
+    animation = new SGFlashAnimation(node);
+  } else if (!strcmp("dist-scale", type)) {
+    animation = new SGDistScaleAnimation(node);
   } else {
     animation = new SGNullAnimation(node);
     SG_LOG(SG_INPUT, SG_WARN, "Unknown animation type " << type);
@@ -144,7 +170,7 @@ sgMakeAnimation( ssgBranch * model,
   if (name_nodes.size() > 0) {
     object = find_named_node(model, name_nodes[0]->getStringValue());
     if (object == 0) {
-      SG_LOG(SG_INPUT, SG_WARN, "Object " << name_nodes[0]->getStringValue()
+      SG_LOG(SG_INPUT, SG_ALERT, "Object " << name_nodes[0]->getStringValue()
              << " not found");
       delete animation;
       animation = 0;
@@ -163,18 +189,22 @@ sgMakeAnimation( ssgBranch * model,
       const char * name = name_nodes[i]->getStringValue();
       object = find_named_node(model, name);
       if (object == 0) {
-          SG_LOG(SG_INPUT, SG_WARN, "Object " << name << " not found");
+          SG_LOG(SG_INPUT, SG_ALERT, "Object " << name << " not found");
           delete animation;
           animation = 0;
+      } else {
+          ssgBranch * oldParent = object->getParent(0);
+          branch->addKid(object);
+          oldParent->removeKid(object);
       }
-      ssgBranch * oldParent = object->getParent(0);
-      branch->addKid(object);
-      oldParent->removeKid(object);
   }
 
-  animation->init();
-  branch->setUserData(animation);
-  branch->setTravCallback(SSG_CALLBACK_PRETRAV, animation_callback);
+  if ( animation != 0 ) {
+    animation->init();
+    branch->setUserData(animation);
+    branch->setTravCallback(SSG_CALLBACK_PRETRAV, animation_callback);
+    branch->setTravCallback(SSG_CALLBACK_POSTTRAV, restore_callback);
+  }
 }
 
 
@@ -187,17 +217,17 @@ sgMakeAnimation( ssgBranch * model,
 ssgBranch *
 sgLoad3DModel( const string &fg_root, const string &path,
                SGPropertyNode *prop_root,
-               double sim_time_sec )
+               double sim_time_sec, ssgEntity *(*load_panel)(SGPropertyNode *) )
 {
   ssgBranch * model = 0;
   SGPropertyNode props;
 
                                 // Load the 3D aircraft object itself
-  SGPath modelpath = path;
+  SGPath modelpath = path, texturepath = path;
   if ( !ulIsAbsolutePathName( path.c_str() ) ) {
     SGPath tmp = fg_root;
     tmp.append(modelpath.str());
-    modelpath = tmp;
+    modelpath = texturepath = tmp;
   }
 
                                 // Check for an XML wrapper
@@ -206,6 +236,10 @@ sgLoad3DModel( const string &fg_root, const string &path,
     if (props.hasValue("/path")) {
       modelpath = modelpath.dir();
       modelpath.append(props.getStringValue("/path"));
+      if (props.hasValue("/texture-path")) {
+        texturepath = texturepath.dir();
+        texturepath.append(props.getStringValue("/texture-path"));
+      }
     } else {
       if (model == 0)
         model = new ssgBranch;
@@ -215,7 +249,10 @@ sgLoad3DModel( const string &fg_root, const string &path,
                                 // Assume that textures are in
                                 // the same location as the XML file.
   if (model == 0) {
-    ssgTexturePath((char *)modelpath.dir().c_str());
+    if (texturepath.extension() != "")
+          texturepath = texturepath.dir();
+
+    ssgTexturePath((char *)texturepath.c_str());
     model = (ssgBranch *)ssgLoad((char *)modelpath.c_str());
     if (model == 0)
       throw sg_exception("Failed to load 3D model");
@@ -223,6 +260,8 @@ sgLoad3DModel( const string &fg_root, const string &path,
 
                                 // Set up the alignment node
   ssgTransform * alignmainmodel = new ssgTransform;
+  if ( load_panel == 0 )
+    alignmainmodel->setTravCallback( SSG_CALLBACK_PRETRAV, model_filter_callback );
   alignmainmodel->addKid(model);
   sgMat4 res_matrix;
   sgMakeOffsetsMatrix(&res_matrix,
@@ -236,6 +275,18 @@ sgLoad3DModel( const string &fg_root, const string &path,
 
   unsigned int i;
 
+  if ( load_panel ) {
+                                // Load panels
+    vector<SGPropertyNode_ptr> panel_nodes = props.getChildren("panel");
+    for (i = 0; i < panel_nodes.size(); i++) {
+        SG_LOG(SG_INPUT, SG_DEBUG, "Loading a panel");
+        ssgEntity * panel = load_panel(panel_nodes[i]);
+        if (panel_nodes[i]->hasValue("name"))
+            panel->setName((char *)panel_nodes[i]->getStringValue("name"));
+        model->addKid(panel);
+    }
+  }
+
                                 // Load sub-models
   vector<SGPropertyNode_ptr> model_nodes = props.getChildren("model");
   for (i = 0; i < model_nodes.size(); i++) {
@@ -252,7 +303,7 @@ sgLoad3DModel( const string &fg_root, const string &path,
     align->setTransform(res_matrix);
 
     ssgBranch * kid = sgLoad3DModel( fg_root, node->getStringValue("path"),
-                                     prop_root, sim_time_sec );
+                                     prop_root, sim_time_sec, load_panel );
     align->addKid(kid);
     align->setName(node->getStringValue("name", ""));
     model->addKid(align);
@@ -271,5 +322,13 @@ sgLoad3DModel( const string &fg_root, const string &path,
   return alignmainmodel;
 }
 
+bool
+sgSetModelFilter( bool filter )
+{
+  bool old = model_filter;
+  model_filter = filter;
+  return old;
+}
+
 
 // end of model.cxx
