@@ -20,7 +20,7 @@
 // Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 // Boston, MA  02111-1307, USA.
 //
-// $Id: sky.cxx,v 1.14 2004/01/29 18:25:55 ehofman Exp $
+// $Id: sky.cxx,v 1.17 2004/04/04 13:41:53 ehofman Exp $
 
 
 #include <plib/sg.h>
@@ -171,12 +171,6 @@ bool SGSky::reposition( SGSkyState &st, double dt )
 void SGSky::preDraw( float alt, float fog_exp2_density ) {
     ssgCullAndDraw( pre_root );
 
-    // FIXME: This should not be needed, but at this time (08/15/2003)
-    //        certain NVidia drivers don't seem to implement
-    //        glPushAttrib(FG_FOG_BIT) properly. The result is that
-    //        there is not fog when looking at the sun.
-    glFogf ( GL_FOG_DENSITY, fog_exp2_density );
-
     	// if we are closer than this to a cloud layer, don't draw clouds
     static const float slop = 5.0;
     int i;
@@ -207,10 +201,18 @@ void SGSky::preDraw( float alt, float fog_exp2_density ) {
        ++cur_layer_pos;
     }
 
+    // FIXME: This should not be needed, but at this time (08/15/2003)
+    //        certain NVidia drivers don't seem to implement
+    //        glPushAttrib(FG_FOG_BIT) properly. The result is that
+    //        there is not fog when looking at the sun.
+    glFogf ( GL_FOG_DENSITY, fog_exp2_density );
+}
+
+void SGSky::drawUpperClouds( ) {
     // draw the cloud layers that are above us, top to bottom
-    for ( i = (int)cloud_layers.size() - 1; i >= cur_layer_pos; --i ) {
+    for ( int i = (int)cloud_layers.size() - 1; i >= cur_layer_pos; --i ) {
         if ( i != in_cloud ) {
-            cloud_layers[i]->draw();
+            cloud_layers[i]->draw( false );
         }
     }
 }
@@ -218,13 +220,12 @@ void SGSky::preDraw( float alt, float fog_exp2_density ) {
 
 // draw translucent clouds ... do this after you've drawn all the
 // oapaque elements of your scene.
-void SGSky::postDraw( float alt ) {
+void SGSky::drawLowerClouds() {
 
     // draw the cloud layers that are below us, bottom to top
-
     for ( int i = 0; i < cur_layer_pos; ++i ) {
         if ( i != in_cloud ) {
-            cloud_layers[i]->draw();
+            cloud_layers[i]->draw( true );
         }
     }
 }
