@@ -7,14 +7,11 @@
 
 #include "serial.hxx"
 
-#if !defined(SG_HAVE_NATIVE_SGI_COMPILERS)
 SG_USING_STD(cout);
 SG_USING_STD(endl);
-#endif
-
 
 int main () {
-    FGSerialPort port;
+    SGSerialPort port;
     string value;
     bool result;
 
